==> apache-doris-2.0.13-src.tar.gz: apache-doris-2.0.13-src/be/src/olap/partial_update_info.h <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#pragma once

#include "olap/tablet_schema.h"
#include "util/string_util.h"

namespace doris {

struct PartialUpdateInfo {
    void init(const TabletSchema& tablet_schema, bool partial_update,
              const std::set<string>& partial_update_cols, bool is_strict_mode,
              int64_t timestamp_ms, const std::string& timezone) {
        is_partial_update = partial_update;
        partial_update_input_columns = partial_update_cols;

        this->timestamp_ms = timestamp_ms;
        this->timezone = timezone;
        missing_cids.clear();
        update_cids.clear();
        for (auto i = 0; i < tablet_schema.num_columns(); ++i) {
            auto tablet_column = tablet_schema.column(i);
            if (!partial_update_input_columns.contains(tablet_column.name())) {
                missing_cids.emplace_back(i);
                if (!tablet_column.has_default_value() && !tablet_column.is_nullable()) {
                    can_insert_new_rows_in_partial_update = false;
                }
            } else {
                update_cids.emplace_back(i);
            }
        }
        this->is_strict_mode = is_strict_mode;
        _generate_default_values_for_missing_cids(tablet_schema);
    }

private:
    void _generate_default_values_for_missing_cids(const TabletSchema& tablet_schema) {
        for (auto i = 0; i < missing_cids.size(); ++i) {
            auto cur_cid = missing_cids[i];
            const auto& column = tablet_schema.column(cur_cid);
            if (column.has_default_value()) {
                std::string default_value;
                if (UNLIKELY(tablet_schema.column(cur_cid).type() ==
                                     FieldType::OLAP_FIELD_TYPE_DATETIMEV2 &&
                             to_lower(tablet_schema.column(cur_cid).default_value())
                                             .find(to_lower("CURRENT_TIMESTAMP")) !=
                                     std::string::npos)) {
                    vectorized::DateV2Value<vectorized::DateTimeV2ValueType> dtv;
                    dtv.from_unixtime(timestamp_ms / 1000, timezone);
                    default_value = dtv.debug_string();
                } else {
                    default_value = tablet_schema.column(cur_cid).default_value();
                }
                default_values.emplace_back(default_value);
            } else {
                // place an empty string here
                default_values.emplace_back();
            }
        }
        CHECK_EQ(missing_cids.size(), default_values.size());
    }

public:
    bool is_partial_update {false};
    std::set<std::string> partial_update_input_columns;
    std::vector<uint32_t> missing_cids;
    std::vector<uint32_t> update_cids;
    // if key not exist in old rowset, use default value or null value for the unmentioned cols
    // to generate a new row, only available in non-strict mode
    bool can_insert_new_rows_in_partial_update {true};
    bool is_strict_mode {false};
    int64_t timestamp_ms {0};
    std::string timezone;

    // default values for missing cids
    std::vector<std::string> default_values;
};
} // namespace doris

==> apache-doris-2.0.15.1-src.tar.gz: apache-doris-2.0.15.1-src/be/src/olap/partial_update_info.h <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#pragma once
#include <cstdint>
#include <set>
#include <string>
#include <vector>

namespace doris {
class TabletSchema;
class PartialUpdateInfoPB;

struct PartialUpdateInfo {
    void init(const TabletSchema& tablet_schema, bool partial_update,
              const std::set<std::string>& partial_update_cols, bool is_strict_mode,
              int64_t timestamp_ms, int32_t nano_seconds, const std::string& timezone,
              int64_t cur_max_version = -1);
    void to_pb(PartialUpdateInfoPB* partial_update_info) const;
    void from_pb(PartialUpdateInfoPB* partial_update_info);
    std::string summary() const;

private:
    void _generate_default_values_for_missing_cids(const TabletSchema& tablet_schema);

public:
    bool is_partial_update {false};
    int64_t max_version_in_flush_phase {-1};
    std::set<std::string> partial_update_input_columns;
    std::vector<uint32_t> missing_cids;
    std::vector<uint32_t> update_cids;
    // if key not exist in old rowset, use default value or null value for the unmentioned cols
    // to generate a new row, only available in non-strict mode
    bool can_insert_new_rows_in_partial_update {true};
    bool is_strict_mode {false};
    int64_t timestamp_ms {0};
    int32_t nano_seconds {0};
    std::string timezone;

    // default values for missing cids
    std::vector<std::string> default_values;
};
} // namespace doris
